@@ -33,20 +33,21 @@
 Frustum::Frustum() {
 }
 
-Frustum::Frustum(Camera& camera) {
-    update(camera);
+Frustum::Frustum(const vec3& source, const vec3& target, const vec3& up, float fov, float near_distance, float far_distance) {
+    update(source, target, up, fov, near_distance, far_distance);
 }
 
-void Frustum::update(Camera& camera) {
-    updatePerspective(camera);
-    updateView(camera);
+void Frustum::update(const vec3& source, const vec3& target, const vec3& up, float fov, float near_distance, float far_distance) {
+    updatePerspective(fov, near_distance, far_distance);
+    updateView(source, target, up);
 }
 
-void Frustum::updatePerspective(Camera& camera) {
+void Frustum::updatePerspective(float fov, float near_distance, float far_distance) {
+
+    this->near_distance = near_distance;
+    this->far_distance  = far_distance;
+
     view_ratio    = (float) display.width / (float) display.height;
-    fov           = camera.getFov();
-    near_distance = camera.getZNear();
-    far_distance  = camera.getZFar();
 
     float toa = (float) tan(fov * 0.5 * DEGREES_TO_RADIANS);
 
@@ -56,35 +57,30 @@ void Frustum::updatePerspective(Camera& camera) {
     far_half_width   = far_half_height * view_ratio;
 }
 
-void Frustum::updateView(Camera& camera) {
-    vec3f source = camera.getPos();
-    vec3f target = camera.getTarget();
-    vec3f up     = camera.getUp();
+void Frustum::updateView(const vec3& source, const vec3& target, const vec3& up) {
 
-    vec3f view_ray = target - source;
-    view_ray.normalize();
+    vec3 view_ray = normalise(target - source);
 
-    vec3f horiz_normal = view_ray.cross(up);
-    horiz_normal.normalize();
+    vec3 horiz_normal = normalise(glm::cross(view_ray,up));
 
-    vec3f vert_normal = horiz_normal.cross(view_ray);
+    vec3 vert_normal = glm::cross(horiz_normal, view_ray);
 
     //calculate the positions of the 8 points that make up
     //the viewing frustum and then use them to create the 6 planes
 
-    vec3f near_centre = source + view_ray * near_distance;
-    vec3f far_centre  = source + view_ray * far_distance;
+    vec3 near_centre = source + view_ray * near_distance;
+    vec3 far_centre  = source + view_ray * far_distance;
 
-    vec3f near_horiz_offset = horiz_normal * near_half_width;
-    vec3f near_vert_offset  = vert_normal  * near_half_height;
+    vec3 near_horiz_offset = horiz_normal * near_half_width;
+    vec3 near_vert_offset  = vert_normal  * near_half_height;
 
     near_top_left     = near_centre + near_vert_offset - near_horiz_offset;
     near_top_right    = near_centre + near_vert_offset + near_horiz_offset;
     near_bottom_left  = near_centre - near_vert_offset - near_horiz_offset;
     near_bottom_right = near_centre - near_vert_offset + near_horiz_offset;
 
-    vec3f far_horiz_offset = horiz_normal * far_half_width;
-    vec3f far_vert_offset  = vert_normal  * far_half_height;
+    vec3 far_horiz_offset = horiz_normal * far_half_width;
+    vec3 far_vert_offset  = vert_normal  * far_half_height;
 
     far_top_left     = far_centre + far_vert_offset - far_horiz_offset;
     far_top_right    = far_centre + far_vert_offset + far_horiz_offset;
@@ -110,7 +106,7 @@ void Frustum::updateView(Camera& camera) {
     planes[5] = Plane(far_top_right,     far_top_left,      far_bottom_left);
 }
 
-bool Frustum::contains(const vec3f& p) const {
+bool Frustum::contains(const vec3& p) const {
 
     for(int i=0; i < 6; i++) {
         float dist = planes[i].distance(p);
@@ -123,7 +119,7 @@ bool Frustum::contains(const vec3f& p) const {
 
 bool Frustum::intersects(const Bounds3D& bounds) const {
 
-    vec3f corner;
+    vec3 corner;
 
     for(int i=0; i<6; i++) {
 
@@ -140,7 +136,7 @@ bool Frustum::intersects(const Bounds3D& bounds) const {
 
 bool Frustum::intersects(const Bounds2D& bounds, float z) const {
 
-    vec3f corner;
+    vec3 corner;
 
     for(int i=0; i<6; i++) {
 
