@@ -172,9 +172,7 @@ PPMExporter::PPMExporter(std::string outputfile) {
     }
 
     //write header
-    sprintf(ppmheader, "P6\n# Generated by %s\n%d %d\n255\n",
-        gSDLAppTitle.c_str(), display.width, display.height
-    );
+    sprintf(ppmheader, "P6\n%d %d 255\n", display.width, display.height);
 }
 
 PPMExporter::~PPMExporter() {
