@@ -170,7 +170,6 @@ void ShaderPart::preprocess() {
     std::stringstream in(raw_source);
 
     std::string line;
-    bool skipdef = false;
 
     while( std::getline(in,line) ) {
         applySubstitutions(line);
@@ -242,17 +241,11 @@ float& FloatShaderUniform::getValue() {
 }
 
 void FloatShaderUniform::write(std::string& content) const {
-
-    char buff[256];
-
     if(baked) {
-//        snprintf(buff, 256, "const %s %s = %e;\n", type_name.c_str(), name.c_str(), value);
-        snprintf(buff, 256, "#define %s %e\n", name.c_str(), value);
+        content.append(str(boost::format("#define %s %e\n") % name % value));
     } else {
-        snprintf(buff, 256, "uniform %s %s;\n", type_name.c_str(), name.c_str());
+        content.append(str(boost::format("uniform %s %s;\n") % type_name % name));
     }
-
-    content += buff;
 }
 
 //IntShaderUniform
@@ -274,16 +267,11 @@ int& IntShaderUniform::getValue() {
 }
 
 void IntShaderUniform::write(std::string& content) const {
-
-    char buff[256];
-
     if(baked) {
-        snprintf(buff, 256, "#define %s %d\n", name.c_str(), value);
+        content.append(str(boost::format("#define %s %d\n") % name % value));
     } else {
-        snprintf(buff, 256, "uniform %s %s;\n", type_name.c_str(), name.c_str());
+        content.append(str(boost::format("uniform %s %s;\n") % type_name % name));
     }
-
-    content += buff;
 }
 
 //BoolShaderUniform
@@ -305,16 +293,11 @@ bool& BoolShaderUniform::getValue() {
 }
 
 void BoolShaderUniform::write(std::string& content) const {
-
-    char buff[256];
-
     if(baked) {
-        snprintf(buff, 256, "#define %s %s\n", name.c_str(), value ? "true" : "false");
+        content.append(str(boost::format("#define %s %s\n") % name % (value ? "true" : "false")));
     } else {
-        snprintf(buff, 256, "uniform %s %s;\n", type_name.c_str(), name.c_str());
+        content.append(str(boost::format("uniform %s %s;\n") % type_name % name));
     }
-
-    content += buff;
 }
 
 //Sampler1DShaderUniform
@@ -335,9 +318,7 @@ void Sampler1DShaderUniform::setBaked(bool baked) {
 }
 
 void Sampler1DShaderUniform::write(std::string& content) const {
-    char buff[256];
-    snprintf(buff, 256, "uniform %s %s;\n", type_name.c_str(), name.c_str());
-    content += buff;
+    content.append(str(boost::format("uniform %s %s;\n") % type_name % name));
 }
 
 int& Sampler1DShaderUniform::getValue() {
@@ -367,9 +348,33 @@ void Sampler2DShaderUniform::setBaked(bool baked) {
 }
 
 void Sampler2DShaderUniform::write(std::string& content) const {
-    char buff[256];
-    snprintf(buff, 256, "uniform %s %s;\n", type_name.c_str(), name.c_str());
-    content += buff;
+    content.append(str(boost::format("uniform %s %s;\n") % type_name % name));
+}
+
+//Sampler3DShaderUniform
+
+Sampler3DShaderUniform::Sampler3DShaderUniform(AbstractShader* shader, const std::string& name, int value) :
+    value(value), ShaderUniform(shader, name, SHADER_UNIFORM_SAMPLER_3D, "sampler3D") {
+}
+
+void Sampler3DShaderUniform::setValue(int value) {
+    if(baked && this->value == value) return;
+
+    this->value = value;
+    modified = true;
+    initialized = true;
+}
+
+int& Sampler3DShaderUniform::getValue() {
+    return value;
+}
+
+//cant be baked?
+void Sampler3DShaderUniform::setBaked(bool baked) {
+}
+
+void Sampler3DShaderUniform::write(std::string& content) const {
+    content.append(str(boost::format("uniform %s %s;\n") % type_name % name));
 }
 
 
@@ -392,16 +397,11 @@ vec2& Vec2ShaderUniform::getValue() {
 }
 
 void Vec2ShaderUniform::write(std::string& content) const {
-
-    char buff[256];
-
     if(baked) {
-        snprintf(buff, 256, "#define %s vec2(%e, %e)\n", name.c_str(), value.x, value.y);
+        content.append(str(boost::format("#define %s vec2(%e, %e)\n") % name % value.x % value.y));
     } else {
-        snprintf(buff, 256, "uniform %s %s;\n", type_name.c_str(), name.c_str());
+        content.append(str(boost::format("uniform %s %s;\n") % type_name % name));
     }
-
-    content += buff;
 }
 
 //Vec3ShaderUniform
@@ -424,16 +424,11 @@ vec3& Vec3ShaderUniform::getValue() {
 }
 
 void Vec3ShaderUniform::write(std::string& content) const {
-
-    char buff[256];
-
     if(baked) {
-        snprintf(buff, 256, "#define %s vec3(%e, %e, %e)\n", name.c_str(), value.x, value.y, value.z);
+        content.append(str(boost::format("#define %s vec3(%e, %e, %e)\n") % name % value.x % value.y % value.z));
     } else {
-        snprintf(buff, 256, "uniform %s %s;\n", type_name.c_str(), name.c_str());
+        content.append(str(boost::format("uniform %s %s;\n") % type_name % name));
     }
-
-    content += buff;
 }
 
 //Vec4ShaderUniform
@@ -455,16 +450,11 @@ vec4& Vec4ShaderUniform::getValue() {
 }
 
 void Vec4ShaderUniform::write(std::string& content) const {
-
-    char buff[256];
-
     if(baked) {
-        snprintf(buff, 256, "#define %s vec4(%e, %e, %e, %e)\n", name.c_str(), value.x, value.y, value.z, value.w);
+        content.append(str(boost::format("#define %s vec4(%e, %e, %e, %e)\n") % name % value.x % value.y % value.z % value.w));
     } else {
-        snprintf(buff, 256, "uniform %s %s;\n", type_name.c_str(), name.c_str());
+        content.append(str(boost::format("uniform %s %s;\n") % type_name % name));
     }
-
-    content += buff;
 }
 
 //Mat3ShaderUniform
@@ -486,20 +476,15 @@ mat3& Mat3ShaderUniform::getValue() {
 }
 
 void Mat3ShaderUniform::write(std::string& content) const {
-
-    char buff[1024];
-
     if(baked) {
-        snprintf(buff, 1024, "#define %s mat3(%e, %e, %e, %e, %e, %e, %e, %e, %e)\n", name.c_str(),
-                value[0][0], value[0][1], value[0][2],
-                value[1][0], value[1][1], value[1][2],
-                value[2][0], value[2][1], value[2][2]);
+        content.append(str(boost::format("#define %s mat3(%e, %e, %e, %e, %e, %e, %e, %e, %e)\n") % name
+               % value[0][0] % value[0][1] % value[0][2]
+               % value[1][0] % value[1][1] % value[1][2]
+               % value[2][0] % value[2][1] % value[2][2]));
 
     } else {
-        snprintf(buff, 1024, "uniform %s %s;\n", type_name.c_str(), name.c_str());
+        content.append(str(boost::format("uniform %s %s;\n") % type_name % name));
     }
-
-    content += buff;
 }
 
 //Mat4ShaderUniform
@@ -521,22 +506,132 @@ mat4& Mat4ShaderUniform::getValue() {
 }
 
 void Mat4ShaderUniform::write(std::string& content) const {
-
-    char buff[1024];
-
     if(baked) {
-        snprintf(buff, 1024, "#define %s mat4(%e, %e, %e, %e, %e, %e, %e, %e, %e, %e, %e, %e, %e, %e, %e, %e)\n", name.c_str(),
-                value[0][0], value[0][1], value[0][2], value[0][3],
-                value[1][0], value[1][1], value[1][2], value[1][3],
-                value[2][0], value[2][1], value[2][2], value[2][3],
-                value[3][0], value[3][1], value[3][2], value[3][3]);
-    } else {
-        snprintf(buff, 1024, "uniform %s %s;\n", type_name.c_str(), name.c_str());
-    }
+        content.append(str(boost::format("#define %s mat4(%e, %e, %e, %e, %e, %e, %e, %e, %e, %e, %e, %e, %e, %e, %e, %e)\n") % name
+               % value[0][0] % value[0][1] % value[0][2] % value[0][3]
+               % value[1][0] % value[1][1] % value[1][2] % value[1][3]
+               % value[2][0] % value[2][1] % value[2][2] % value[2][3]
+               % value[3][0] % value[3][1] % value[3][2] % value[3][3]));
 
-    content += buff;
+    } else {
+        content.append(str(boost::format("uniform %s %s;\n") % type_name % name));
+    }
 }
 
+//IntegerArrayShaderUniform
+
+IntegerArrayShaderUniform::IntegerArrayShaderUniform(AbstractShader* shader, const std::string& name, size_t length) :
+    length(length), ShaderUniform(shader, name, SHADER_UNIFORM_INT_ARRAY, "int") {
+}
+
+IntegerArrayShaderUniform::~IntegerArrayShaderUniform() {
+}
+
+const std::vector<int>& IntegerArrayShaderUniform::getValue() {
+    return value;
+}
+
+size_t IntegerArrayShaderUniform::getLength() const {
+    return value.size();
+}
+
+void IntegerArrayShaderUniform::copyValue(const std::vector<int>& value) {
+    this->value = value;
+}
+
+void IntegerArrayShaderUniform::setValue(const std::vector<int>& value) {
+    if(baked) {
+        bool match = true;
+
+        for(size_t i=0;i<length;i++) {
+            if(value[i] != this->value[i]) {
+                match = false;
+                break;
+            }
+        }
+
+        if(match) return;
+    }
+
+    copyValue(value);
+
+    modified = true;
+    initialized = true;
+}
+
+void IntegerArrayShaderUniform::write(std::string& content) const {
+    if(baked) {
+        content.append(str(boost::format("%s[%d] %s = %s[] (\n") % type_name % length % name % type_name));
+
+        for(size_t i=0; i<length; i++) {
+            content.append(str(boost::format("    %d") % value[i]));
+
+            if(i<length-1) content += ",\n";
+            else           content += "\n);\n";
+        }
+
+    } else {
+        content.append(str(boost::format("uniform %s %s[%d];\n") % type_name % name % length));
+    }
+}
+
+
+//FloatArrayShaderUniform
+
+FloatArrayShaderUniform::FloatArrayShaderUniform(AbstractShader* shader, const std::string& name, size_t length) :
+    length(length), ShaderUniform(shader, name, SHADER_UNIFORM_FLOAT_ARRAY, "float") {
+}
+
+FloatArrayShaderUniform::~FloatArrayShaderUniform() {
+}
+
+const std::vector<float>& FloatArrayShaderUniform::getValue() {
+    return value;
+}
+
+size_t FloatArrayShaderUniform::getLength() const {
+    return length;
+}
+
+void FloatArrayShaderUniform::copyValue(const std::vector<float>& value) {
+    this->value = value;
+}
+
+void FloatArrayShaderUniform::setValue(const std::vector<float>& value) {
+    if(baked) {
+        bool match = true;
+
+        for(size_t i=0;i<length;i++) {
+            if(value[i] != this->value[i]) {
+                match = false;
+                break;
+            }
+        }
+
+        if(match) return;
+    }
+
+    copyValue(value);
+
+    modified = true;
+    initialized = true;
+}
+
+void FloatArrayShaderUniform::write(std::string& content) const {
+    if(baked) {
+        content.append(str(boost::format("%s[%d] %s = %s[] (\n") % type_name % length % name % type_name));
+
+        for(size_t i=0; i<length; i++) {
+            content.append(str(boost::format("    %e") % value[i]));
+
+            if(i<length-1) content += ",\n";
+            else           content += "\n);\n";
+        }
+
+    } else {
+        content.append(str(boost::format("uniform %s %s[%d];\n") % type_name % name % length));
+    }
+}
 
 //Vec2ArrayShaderUniform
 
@@ -611,13 +706,9 @@ void Vec2ArrayShaderUniform::setValue(const std::vector<vec2>& value) {
 
 void Vec2ArrayShaderUniform::write(std::string& content) const {
 
-    char buff[1024];
-
     if(baked) {
         content.append(str(boost::format("%s[%d] %s = %s[] (\n") % type_name % length % name % type_name));
 
-        content += buff;
-
         for(size_t i=0; i<length; i++) {
             content.append(str(boost::format("    %s(%e, %e)") % type_name % value[i].x % value[i].y));
 
@@ -703,8 +794,6 @@ void Vec3ArrayShaderUniform::setValue(const std::vector<vec3>& value) {
 
 void Vec3ArrayShaderUniform::write(std::string& content) const {
 
-    char buff[1024];
-
     if(baked) {
         content.append(str(boost::format("%s[%d] %s = %s[] (\n") % type_name % length % name % type_name));
 
@@ -792,9 +881,6 @@ void Vec4ArrayShaderUniform::setValue(const std::vector<vec4>& value) {
 }
 
 void Vec4ArrayShaderUniform::write(std::string& content) const {
-
-    char buff[1024];
-
     if(baked) {
         content.append(str(boost::format("%s[%d] %s = %s[] (\n") % type_name % length % name % type_name));
 
@@ -824,20 +910,18 @@ void AbstractShaderPass::showContext(std::string& context, int line_no, int amou
     std::stringstream in(shader_object_source);
 
     int i = 1;
-    char line_detail[1024];
     std::string line;
 
     while( std::getline(in,line) ) {
 
         if(i==line_no || (i<line_no && i+amount>=line_no) || (i>line_no && i-amount<=line_no)) {
-            snprintf(line_detail, 1024, "%s%4d | %s\n", (i==line_no ? "-> ": "   "), i, line.c_str());
-            context += line_detail;
+            context.append(str(boost::format("%s%4d | %s\n") % (i==line_no ? "-> ": "   ") % i % line));
         }
         i++;
     }
 }
 
-bool AbstractShaderPass::errorContext(const char* log_message, std::string& context) {
+bool AbstractShaderPass::errorContext(const std::string& log_message, std::string& context) {
 
     std::vector<std::string> matches;
 
@@ -896,7 +980,11 @@ ShaderUniform* AbstractShaderPass::addArrayUniform(const std::string& name, cons
 
     if((uniform = parent->getUniform(name)) == 0) {
 
-        if(type == "vec2") {
+        if(type == "int") {
+            uniform = new IntegerArrayShaderUniform(parent, name, length);
+        } else if(type == "float") {
+            uniform = new FloatArrayShaderUniform(parent, name, length);
+        } else if(type == "vec2") {
             uniform = new Vec2ArrayShaderUniform(parent, name, length);
         } else if(type == "vec3") {
             uniform = new Vec3ArrayShaderUniform(parent, name, length);
@@ -932,6 +1020,8 @@ ShaderUniform* AbstractShaderPass::addUniform(const std::string& name, const std
             uniform = new Sampler1DShaderUniform(parent, name);
         } else if(type == "sampler2D") {
             uniform = new Sampler2DShaderUniform(parent, name);
+        } else if(type == "sampler3D") {
+            uniform = new Sampler3DShaderUniform(parent, name);
         } else if(type == "vec2") {
             uniform = new Vec2ShaderUniform(parent, name);
         } else if(type == "vec3") {
@@ -1050,6 +1140,7 @@ void AbstractShader::setDefaults() {
     vertex_shader   = 0;
     fragment_shader = 0;
     geometry_shader = 0;
+    compute_shader  = 0;
     program = 0;
     dynamic_compile = false;
 }
@@ -1061,21 +1152,25 @@ void AbstractShader::clear() {
         delete it->second;
     }
     uniforms.clear();
+    uniform_list.clear();
 
     if(vertex_shader != 0)   delete vertex_shader;
     if(geometry_shader != 0) delete geometry_shader;
     if(fragment_shader != 0) delete fragment_shader;
+    if(compute_shader != 0) delete compute_shader;
 
     vertex_shader   = 0;
     geometry_shader = 0;
     fragment_shader = 0;
+    compute_shader  = 0;
 }
 
 bool AbstractShader::isEmpty() {
 
     if(   (!vertex_shader   || vertex_shader->isEmpty())
        && (!fragment_shader || fragment_shader->isEmpty())
-       && (!geometry_shader || geometry_shader->isEmpty())) {
+       && (!geometry_shader || geometry_shader->isEmpty())
+       && (!compute_shader  || compute_shader->isEmpty())) {
         return true;
     }
 
@@ -1124,6 +1219,7 @@ void AbstractShader::addUniform(ShaderUniform* uniform) {
     }
 
     uniforms[uniform->getName()] = uniform;
+    uniform_list.push_back(uniform);
 }
 
 ShaderUniform* AbstractShader::getUniform(const std::string& name) {
@@ -1225,6 +1321,14 @@ void AbstractShader::setSampler2D (const std::string& name, int value) {
     ((Sampler2DShaderUniform*)uniform)->setValue(value);
 }
 
+void AbstractShader::setSampler3D (const std::string& name, int value) {
+    ShaderUniform* uniform = getUniform(name);
+
+    if(!uniform || uniform->getType() != SHADER_UNIFORM_SAMPLER_3D) return;
+
+    ((Sampler3DShaderUniform*)uniform)->setValue(value);
+}
+
 void AbstractShader::setFloat(const std::string& name, float value) {
     ShaderUniform* uniform = getUniform(name);
 
@@ -1249,6 +1353,22 @@ void AbstractShader::setVec3 (const std::string& name, const vec3& value) {
     ((Vec3ShaderUniform*)uniform)->setValue(value);
 }
 
+void AbstractShader::setFloatArray(const std::string& name, std::vector<float>& value) {
+    ShaderUniform* uniform = getUniform(name);
+
+    if(!uniform || uniform->getType() != SHADER_UNIFORM_FLOAT_ARRAY) return;
+
+    ((FloatArrayShaderUniform*)uniform)->setValue(value);
+}
+
+void AbstractShader::setIntegerArray (const std::string& name, std::vector<int>& value) {
+    ShaderUniform* uniform = getUniform(name);
+
+    if(!uniform || uniform->getType() != SHADER_UNIFORM_INT_ARRAY) return;
+
+    ((IntegerArrayShaderUniform*)uniform)->setValue(value);
+}
+
 void AbstractShader::setVec2Array (const std::string& name, vec2* value) {
     ShaderUniform* uniform = getUniform(name);
 
@@ -1336,10 +1456,8 @@ void AbstractShader::setBakedUniforms(bool baked) {
 }
 
 
-void AbstractShader::getUniforms(std::list<ShaderUniform*>& uniform_list) {
-    for(std::map<std::string, ShaderUniform*>::iterator it= uniforms.begin(); it!=uniforms.end();it++) {
-        uniform_list.push_back(it->second);
-    }
+const std::list<ShaderUniform*>& AbstractShader::getUniforms() {
+    return uniform_list;
 }
 
 void AbstractShader::applyUniforms() {
