@@ -833,7 +833,7 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
 #else
             dirfile = p.filename().string();
 #endif
-            std::string file_ext = extension(p);
+            std::string file_ext = p.extension().string();
             boost::algorithm::to_lower(file_ext);
 
             if(file_ext != ".jpg" && file_ext != ".jpeg" && file_ext != ".png") continue;
